%%% Make sure you have the particle tracking code from the Kilfoil group
%%% in your PATH somewhere: http://people.umass.edu/kilfoil/downloads.html
%%%
%%% The matrix ParticleDataRaw was created by using feature2D from the same
%%% code on a series of grayscale images captured by a camera with 640*480
%%% resolution. Since the raw data is over 8Gb in size, it is only possible
%%% to provide the extracted particle positions as opposed to the complete
%%% images.

%% Look for particle position data in memory. If it's not there, load it.

if ~exist('ParticleDataRaw', 'var')
    try
        load('ParticleDataRaw.mat')
    catch
        error('ParticleDataRaw not found!')
    end
end
FrameNumber = max(ParticleDataRaw(:,3));
ParticleData = ParticleDataRaw;

%% Identify each particle
% Particles must move less than 0.5 pixels, tracks must be at least 100
% frames long and a maximum of 10 frames can be skipped
ParticleData = trackmem([ParticleData(:,1:3),zeros(size(ParticleData,1),1)],0.5,2,100,10); % add in extra column to play nice with trackmem...
ParticleData = ParticleData(:, [1:size(ParticleData,2)-2,size(ParticleData,2)]); % then remove it

ParticleData = sortrows(ParticleData, [4,3]);

%% Average particles per frame
ParticlesPerFrame = NaN(FrameNumber,1);
for i = 0:FrameNumber-1
    ParticlesPerFrame(i+1) = sum(ParticleData(:,3) == i);
end

%% Get dt = 1 data for whole dataset
OffsetData = NaN(FrameNumber,1);
StartParticles = ParticleData(ParticleData(:,3)==0,:);
for Index = 1:FrameNumber-1
    EndParticles = ParticleData(ParticleData(:,3)==Index,:);
    CommonParticles = StartParticles(ismember(StartParticles(:,4), ...
        EndParticles(:,4)),4);
    if length(CommonParticles) > size(OffsetData,2) % resize to match number of particles
        OffsetData = [OffsetData, NaN(size(OffsetData,1), length(CommonParticles)-size(OffsetData,2))];
    end
    ParticleDataCommon = sortrows([StartParticles(ismember(...
        StartParticles(:,4), CommonParticles),:); EndParticles(ismember(...
        EndParticles(:,4), CommonParticles),:)], [3,4]);
    for ParticleNumber = 2:length(CommonParticles)
        Shifts = ParticleDataCommon(1+length(CommonParticles):...
            ParticleNumber+length(CommonParticles), 1:2) - ...
            ParticleDataCommon(1:ParticleNumber, 1:2);
        Shifts = Shifts - repmat(nanmean(Shifts,1), [size(Shifts,1), 1]); % Correct for vibration
        OffsetData(Index, ParticleNumber) = nanmean(Shifts(:,1).^2+Shifts(:,2).^2);
    end
    StartParticles = EndParticles; % reassign for next loop iteration
end

% Fit the amplitude of the trendline
NumParticleIndex = 2:size(OffsetData,2);
OffsetMean = nanmean(OffsetData(:,2:end), 1);
FitOpts = fitoptions('Method','NonlinearLeastSquares',...
               'Lower',[0],...
               'Upper',[Inf],...
               'Startpoint',[3.9e-3]);
FitFunc = fittype('a*(1-1/x)', 'options', FitOpts);
FitObj = fit(NumParticleIndex',OffsetMean',FitFunc);
clear FitFunc FitOpts OffsetMean NumParticleIndex

fig3 = figure(3);
set(fig3,'Position',[100 100 450 250])
scatter(1:size(OffsetData,2), nanmean(OffsetData,1))
hold on
plot(2:size(OffsetData,2), FitObj.a*(1-(1./(2:size(OffsetData,2)))), 'r')
line(xlim,[FitObj.a, FitObj.a], 'LineStyle', '--', 'Color', 'g')
xlim([0,85])
ylabel('Measured MSD / pixels^2 frame^{-1}')
xlabel('Number of tracked particles')
legend('Averaged Measured MSDs','A(1-1/n)','A', 'Location', 'SouthEast')
set(gcf, 'Color', 'w');
hold off